"""
Django system checks for Alliance Auth
"""

from sqlite3.dbapi2 import sqlite_version_info
from typing import List

from celery import current_app
from packaging.version import InvalidVersion, Version as Pep440Version

from django import db
from django.conf import settings
from django.core.checks import CheckMessage, Error, Warning, register
from django.utils import timezone

from allianceauth.utils.cache import get_redis_client

"""
A = System Packages
B = Configuration
"""


@register()
def django_settings(app_configs, **kwargs) -> List[CheckMessage]:
    """
    Check that Django settings are correctly configured

    :param app_configs:
    :type app_configs:
    :param kwargs:
    :type kwargs:
    :return:
    :rtype:
    """

    errors: List[CheckMessage] = []

    # Check for SITE_URL
    if hasattr(settings, "SITE_URL"):
        # Check if SITE_URL is empty
        if settings.SITE_URL == "":
            errors.append(
                Error(
                    msg="'SITE_URL' is empty.",
                    hint="Make sure to set 'SITE_URL' to the URL of your Auth instance. (Without trailing slash)",
                    id="allianceauth.checks.B011",
                )
            )
        # Check if SITE_URL has a trailing slash
        elif settings.SITE_URL[-1] == "/":
            errors.append(
                Warning(
                    msg="'SITE_URL' has a trailing slash. This may lead to incorrect links being generated by Auth.",
                    hint="",
                    id="allianceauth.checks.B005",
                )
            )
    # SITE_URL not found
    else:
        errors.append(
            Error(
                msg="No 'SITE_URL' found is settings. This may lead to incorrect links being generated by Auth or Errors in 3rd party modules.",
                hint="",
                id="allianceauth.checks.B006",
            )
        )

    # Check for CSRF_TRUSTED_ORIGINS
    if hasattr(settings, "CSRF_TRUSTED_ORIGINS") and hasattr(settings, "SITE_URL"):
        # Check if SITE_URL is not in CSRF_TRUSTED_ORIGINS
        if settings.SITE_URL not in settings.CSRF_TRUSTED_ORIGINS:
            errors.append(
                Warning(
                    msg="'SITE_URL' not found in 'CSRF_TRUSTED_ORIGINS'. Auth may not load pages correctly until this is rectified.",
                    hint="",
                    id="allianceauth.checks.B007",
                )
            )
    # CSRF_TRUSTED_ORIGINS not found
    else:
        errors.append(
            Error(
                msg="No 'CSRF_TRUSTED_ORIGINS' found is settings, Auth may not load pages correctly until this is rectified",
                hint="",
                id="allianceauth.checks.B008",
            )
        )

    # Check for ESI_USER_CONTACT_EMAIL
    if hasattr(settings, "ESI_USER_CONTACT_EMAIL"):
        # Check if ESI_USER_CONTACT_EMAIL is empty
        if settings.ESI_USER_CONTACT_EMAIL == "":
            errors.append(
                Error(
                    msg="'ESI_USER_CONTACT_EMAIL' is empty. A valid email is required as maintainer contact for CCP.",
                    hint="",
                    id="allianceauth.checks.B009",
                )
            )
    # ESI_USER_CONTACT_EMAIL not found
    else:
        errors.append(
            Error(
                msg="No 'ESI_USER_CONTACT_EMAIL' found is settings. A valid email is required as maintainer contact for CCP.",
                hint="",
                id="allianceauth.checks.B010",
            )
        )

    return errors


@register()
def system_package_redis(app_configs, **kwargs) -> List[CheckMessage]:
    """
    Check that Redis is a supported version

    :param app_configs:
    :type app_configs:
    :param kwargs:
    :type kwargs:
    :return:
    :rtype:
    """

    allianceauth_redis_install_link = "https://allianceauth.readthedocs.io/en/latest/installation/allianceauth.html#redis-and-other-tools"

    errors: List[CheckMessage] = []

    try:
        # Traditional Redis Version
        redis_version = Pep440Version(get_redis_client().info()["redis_version"])
        # Exit early if Valkey, this will be handled in Valkey checks
        if "valkey_version" in get_redis_client().info():
            return errors
    except InvalidVersion:
        errors.append(Warning("Unable to confirm Redis Version"))

        return errors

    if (
        redis_version.major == 7 and redis_version.minor == 2 and timezone.now() > timezone.datetime(year=2025, month=8, day=31, tzinfo=timezone.utc)
    ):
        errors.append(
            Error(
                msg=f"Redis {redis_version.public} in Security Support only, Updating Suggested",
                hint=allianceauth_redis_install_link,
                id="allianceauth.checks.A001",
            )
        )
    elif redis_version.major == 7 and redis_version.minor == 0:
        errors.append(
            Warning(
                msg=f"Redis {redis_version.public} in Security Support only, Updating Suggested",
                hint=allianceauth_redis_install_link,
                id="allianceauth.checks.A002",
            )
        )
    elif redis_version.major == 6 and redis_version.minor == 2:
        errors.append(
            Warning(
                msg=f"Redis {redis_version.public} in Security Support only, Updating Suggested",
                hint=allianceauth_redis_install_link,
                id="allianceauth.checks.A018",
            )
        )
    elif redis_version.major in [6, 5]:
        errors.append(
            Error(
                msg=f"Redis {redis_version.public} EOL",
                hint=allianceauth_redis_install_link,
                id="allianceauth.checks.A003",
            )
        )

    return errors


@register()
def system_package_valkey(app_configs, **kwargs) -> List[CheckMessage]:
    """
    Check that Valkey is a supported version

    :param app_configs:
    :type app_configs:
    :param kwargs:
    :type kwargs:
    :return:
    :rtype:
    """

    allianceauth_redis_install_link = "https://allianceauth.readthedocs.io/en/latest/installation/allianceauth.html#redis-and-other-tools"

    errors: List[CheckMessage] = []

    try:
        # Valkey has both redis_version and valkey_version in info
        if "valkey_version" in get_redis_client().info():
            valkey_version: Pep440Version = Pep440Version(get_redis_client().info()["valkey_version"])
        else:
            # Just here to typehint the following code, this will return out of the check
            valkey_version = Pep440Version("0.0.0")
            return errors
    except InvalidVersion:
        errors.append(Warning("Unable to confirm Redis Version"))

        return errors

    if valkey_version.major == 7 and valkey_version.minor == 2 and timezone.now() > timezone.datetime(year=2027, month=4, day=16, tzinfo=timezone.utc):
        errors.append(
            Error(
                msg=f"Valkey {valkey_version.public} in Security Support only, Updating Suggested",
                hint=allianceauth_redis_install_link,
                id="allianceauth.checks.A021",
            )
        )
    elif valkey_version.major == 8 and valkey_version.minor == 0 and timezone.now() > timezone.datetime(year=2027, month=9, day=15, tzinfo=timezone.utc):
        errors.append(
            Warning(
                msg=f"Valkey {valkey_version.public} in Security Support only, Updating Suggested",
                hint=allianceauth_redis_install_link,
                id="allianceauth.checks.A022",
            )
        )
    elif valkey_version.major == 8 and valkey_version.minor == 1 and timezone.now() > timezone.datetime(year=2028, month=5, day=31, tzinfo=timezone.utc):
        errors.append(
            Warning(
                msg=f"Valkey {valkey_version.public} in Security Support only, Updating Suggested",
                hint=allianceauth_redis_install_link,
                id="allianceauth.checks.A023",
            )
        )
    elif valkey_version.major == 9 and valkey_version.minor == 0 and timezone.now() > timezone.datetime(year=2028, month=10, day=21, tzinfo=timezone.utc):
        errors.append(
            Warning(
                msg=f"Valkey {valkey_version.public} in Security Support only, Updating Suggested",
                hint=allianceauth_redis_install_link,
                id="allianceauth.checks.A024",
            )
        )
    return errors


@register()
def system_package_mysql(app_configs, **kwargs) -> List[CheckMessage]:
    """
    Check that MySQL is a supported version

    :param app_configs:
    :type app_configs:
    :param kwargs:
    :type kwargs:
    :return:
    :rtype:
    """

    mysql_quick_guide_link = "https://dev.mysql.com/doc/mysql-apt-repo-quick-guide/en/"

    errors: List[CheckMessage] = []

    for connection in db.connections.all():
        if connection.vendor == "mysql":
            try:
                mysql_version = Pep440Version(
                    ".".join(str(i) for i in connection.mysql_version)
                )
            except InvalidVersion:
                errors.append(Warning("Unable to confirm MySQL Version"))

                return errors

            #  MySQL 8
            if mysql_version.major == 8:
                if mysql_version.minor == 4 and timezone.now() > timezone.datetime(
                    year=2032, month=4, day=30, tzinfo=timezone.utc
                ):
                    errors.append(
                        Error(
                            msg=f"MySQL {mysql_version.public} EOL",
                            hint=mysql_quick_guide_link,
                            id="allianceauth.checks.A004",
                        )
                    )
                elif mysql_version.minor == 3:
                    errors.append(
                        Warning(
                            msg=f"MySQL {mysql_version.public} Non LTS",
                            hint=mysql_quick_guide_link,
                            id="allianceauth.checks.A005",
                        )
                    )
                elif mysql_version.minor == 2:
                    errors.append(
                        Warning(
                            msg=f"MySQL {mysql_version.public} Non LTS",
                            hint=mysql_quick_guide_link,
                            id="allianceauth.checks.A006",
                        )
                    )
                elif mysql_version.minor == 1:
                    errors.append(
                        Error(
                            msg=f"MySQL {mysql_version.public} EOL",
                            hint=mysql_quick_guide_link,
                            id="allianceauth.checks.A007",
                        )
                    )
                elif mysql_version.minor == 0 and timezone.now() > timezone.datetime(
                    year=2026, month=4, day=30, tzinfo=timezone.utc
                ):
                    errors.append(
                        Error(
                            msg=f"MySQL {mysql_version.public} EOL",
                            hint=mysql_quick_guide_link,
                            id="allianceauth.checks.A008",
                        )
                    )

            # MySQL below 8
            # This will also catch Mariadb 5.x
            elif mysql_version.major < 8:
                errors.append(
                    Error(
                        msg=f"MySQL or MariaDB {mysql_version.public} EOL",
                        hint=mysql_quick_guide_link,
                        id="allianceauth.checks.A009",
                    )
                )

    return errors


@register()
def system_package_mariadb(app_configs, **kwargs) -> List[CheckMessage]:
    """
    Check that MariaDB is a supported version

    :param app_configs:
    :type app_configs:
    :param kwargs:
    :type kwargs:
    :return:
    :rtype:
    """

    mariadb_download_link = "https://mariadb.org/download/?t=repo-config"

    errors: List[CheckMessage] = []

    for connection in db.connections.all():
        # TODO: Find a way to determine MySQL vs. MariaDB
        if connection.vendor == "mysql":
            try:
                mariadb_version = Pep440Version(
                    ".".join(str(i) for i in connection.mysql_version)
                )
            except InvalidVersion:
                errors.append(Warning("Unable to confirm MariaDB Version"))

                return errors

            # MariaDB 11
            if mariadb_version.major == 11:
                if mariadb_version.minor == 4 and timezone.now() > timezone.datetime(
                    year=2029, month=5, day=19, tzinfo=timezone.utc
                ):
                    errors.append(
                        Error(
                            msg=f"MariaDB {mariadb_version.public} EOL",
                            hint=mariadb_download_link,
                            id="allianceauth.checks.A010",
                        )
                    )
                elif mariadb_version.minor == 2:
                    errors.append(
                        Warning(
                            msg=f"MariaDB {mariadb_version.public} Non LTS",
                            hint=mariadb_download_link,
                            id="allianceauth.checks.A018",
                        )
                    )

                    if timezone.now() > timezone.datetime(
                        year=2024, month=11, day=21, tzinfo=timezone.utc
                    ):
                        errors.append(
                            Error(
                                msg=f"MariaDB {mariadb_version.public} EOL",
                                hint=mariadb_download_link,
                                id="allianceauth.checks.A011",
                            )
                        )
                elif mariadb_version.minor == 1:
                    errors.append(
                        Warning(
                            msg=f"MariaDB {mariadb_version.public} Non LTS",
                            hint=mariadb_download_link,
                            id="allianceauth.checks.A019",
                        )
                    )

                    if timezone.now() > timezone.datetime(
                        year=2024, month=8, day=21, tzinfo=timezone.utc
                    ):
                        errors.append(
                            Error(
                                msg=f"MariaDB {mariadb_version.public} EOL",
                                hint=mariadb_download_link,
                                id="allianceauth.checks.A012",
                            )
                        )
                # Demote versions down here once EOL
                elif mariadb_version.minor in [0, 3]:
                    errors.append(
                        Error(
                            msg=f"MariaDB {mariadb_version.public} EOL",
                            hint=mariadb_download_link,
                            id="allianceauth.checks.A013",
                        )
                    )

            # MariaDB 10
            elif mariadb_version.major == 10:
                if mariadb_version.minor == 11 and timezone.now() > timezone.datetime(
                    year=2028, month=2, day=10, tzinfo=timezone.utc
                ):
                    errors.append(
                        Error(
                            msg=f"MariaDB {mariadb_version.public} EOL",
                            hint=mariadb_download_link,
                            id="allianceauth.checks.A014",
                        )
                    )
                elif mariadb_version.minor == 6 and timezone.now() > timezone.datetime(
                    year=2026, month=7, day=6, tzinfo=timezone.utc
                ):
                    errors.append(
                        Error(
                            msg=f"MariaDB {mariadb_version.public} EOL",
                            hint=mariadb_download_link,
                            id="allianceauth.checks.A0015",
                        )
                    )
                elif mariadb_version.minor == 5 and timezone.now() > timezone.datetime(
                    year=2025, month=6, day=24, tzinfo=timezone.utc
                ):
                    errors.append(
                        Error(
                            msg=f"MariaDB {mariadb_version.public} EOL",
                            hint=mariadb_download_link,
                            id="allianceauth.checks.A016",
                        )
                    )
                # Demote versions down here once EOL
                elif mariadb_version.minor in [0, 1, 2, 3, 4, 7, 9, 10]:
                    errors.append(
                        Error(
                            msg=f"MariaDB {mariadb_version.public} EOL",
                            hint=mariadb_download_link,
                            id="allianceauth.checks.A017",
                        )
                    )

    return errors


@register()
def system_package_sqlite(app_configs, **kwargs) -> List[CheckMessage]:
    """
    Check that SQLite is a supported version

    :param app_configs:
    :type app_configs:
    :param kwargs:
    :type kwargs:
    :return:
    :rtype:
    """

    errors: List[CheckMessage] = []

    for connection in db.connections.all():
        if connection.vendor == "sqlite":
            try:
                sqlite_version = Pep440Version(
                    ".".join(str(i) for i in sqlite_version_info)
                )
            except InvalidVersion:
                errors.append(Warning("Unable to confirm SQLite Version"))

                return errors
            if sqlite_version.major == 3 and sqlite_version.minor < 27:
                errors.append(
                    Error(
                        msg=f"SQLite {sqlite_version.public} Unsupported by Django",
                        hint="https://pkgs.org/download/sqlite3",
                        id="allianceauth.checks.A020",
                    )
                )

    return errors


@register()
def sql_settings(app_configs, **kwargs) -> List[CheckMessage]:
    """
    Check that SQL settings are correctly configured

    :param app_configs:
    :type app_configs:
    :param kwargs:
    :type kwargs:
    :return:
    :rtype:
    """

    errors: List[CheckMessage] = []

    for connection in db.connections.all():
        if connection.vendor == "mysql":
            try:
                if connection.settings_dict["OPTIONS"]["charset"] != "utf8mb4":
                    errors.append(
                        Error(
                            msg=f"SQL Charset is not set to utf8mb4 DB: {connection.alias}",
                            hint="https://gitlab.com/allianceauth/allianceauth/-/commit/89be2456fb2d741b86417e889da9b6129525bec8",
                            id="allianceauth.checks.B001",
                        )
                    )
            except KeyError:
                errors.append(
                    Error(
                        msg=f"SQL Charset is not set to utf8mb4 DB: {connection.alias}",
                        hint="https://gitlab.com/allianceauth/allianceauth/-/commit/89be2456fb2d741b86417e889da9b6129525bec8",
                        id="allianceauth.checks.B001",
                    )
                )

            # This hasn't actually been set on AA yet
            # try:
            #     if (
            #         connection.settings_dict["OPTIONS"]["collation"]
            #         != "utf8mb4_unicode_ci"
            #     ):
            #         errors.append(
            #             Error(
            #                 msg=f"SQL Collation is not set to utf8mb4_unicode_ci DB:{connection.alias}",
            #                 hint="https://gitlab.com/allianceauth/allianceauth/-/commit/89be2456fb2d741b86417e889da9b6129525bec8",
            #                 id="allianceauth.checks.B001",
            #             )
            #         )
            # except KeyError:
            #     errors.append(
            #         Error(
            #             msg=f"SQL Collation is not set to utf8mb4_unicode_ci DB:{connection.alias}",
            #             hint="https://gitlab.com/allianceauth/allianceauth/-/commit/89be2456fb2d741b86417e889da9b6129525bec8",
            #             id="allianceauth.checks.B001",
            #         )
            #     )

        # if connection.vendor == "sqlite":

    return errors


@register()
def celery_settings(app_configs, **kwargs) -> List[CheckMessage]:
    """
    Check that Celery settings are correctly configured

    :param app_configs:
    :type app_configs:
    :param kwargs:
    :type kwargs:
    :return:
    :rtype:
    """

    errors: List[CheckMessage] = []

    try:
        if current_app.conf.broker_transport_options != {
            "priority_steps": [0, 1, 2, 3, 4, 5, 6, 7, 8, 9],
            "queue_order_strategy": "priority",
        }:
            errors.append(
                Error(
                    msg="Celery Priorities are not set correctly",
                    hint="https://gitlab.com/allianceauth/allianceauth/-/commit/8861ec0a61790eca0261f1adc1cc04ca5f243cbc",
                    id="allianceauth.checks.B003",
                )
            )
    except KeyError:
        errors.append(
            Error(
                msg="Celery Priorities are not set",
                hint="https://gitlab.com/allianceauth/allianceauth/-/commit/8861ec0a61790eca0261f1adc1cc04ca5f243cbc",
                id="allianceauth.checks.B003",
            )
        )

    try:
        if not current_app.conf.broker_connection_retry_on_startup:
            errors.append(
                Error(
                    msg="Celery broker_connection_retry_on_startup not set correctly",
                    hint="https://gitlab.com/allianceauth/allianceauth/-/commit/380c41400b535447839e5552df2410af35a75280",
                    id="allianceauth.checks.B004",
                )
            )
    except KeyError:
        errors.append(
            Error(
                msg="Celery broker_connection_retry_on_startup not set",
                hint="https://gitlab.com/allianceauth/allianceauth/-/commit/380c41400b535447839e5552df2410af35a75280",
                id="allianceauth.checks.B004",
            )
        )

    return errors


# IDEAS

# Any other celery things weve manually changed over the years
# I'd be happy to add Community App checks, old versions the owners dont want to support etc.


# Check Default Collation on DB
# Check Charset Collation on all tables



"""
Alliance Auth Middleware
"""

from user_agents import parse


class DeviceDetectionMiddleware:
    """
    Middleware to detect the type of device making the request.
    Sets flags on the request object for easy access in views and templates.

    Flags include:
    - is_mobile: True if the device is a mobile phone.
    - is_tablet: True if the device is a tablet.
    - is_mobile_device: True if the device is either a mobile phone or a tablet.
    - is_touch_capable: True if the device has touch capabilities.
    - is_pc: True if the device is a desktop or laptop computer.
    - is_bot: True if the device is identified as a bot or crawler.
    """

    def __init__(self, get_response):
        """
        Initialize the middleware with the get_response callable.

        :param get_response:
        :type get_response:
        """

        self.get_response = get_response

    def __call__(self, request):
        """
        Process the incoming request to determine if it's from a mobile device.

        This method is called when the middleware is invoked. It inspects the
        `user-agent` header of the incoming HTTP request to determine the type
        of client making the request (e.g., mobile, tablet, PC, bot, etc.).
        Flags are set on the `request` object to indicate the client type.

        :param request: The HTTP request object.
        :type request: HttpRequest
        :return: The HTTP response object after processing the request.
        :rtype: HttpResponse
        """

        # Retrieve the user-agent string from the request headers
        user_agent_string = request.headers.get("user-agent", "")

        # Parse the user-agent string to extract client information
        user_agent = parse(user_agent_string)

        # Set flags on the request object based on the client type
        request.is_mobile = user_agent.is_mobile  # True if the client is a mobile phone
        request.is_tablet = user_agent.is_tablet  # True if the client is a tablet
        request.is_mobile_device = user_agent.is_mobile or user_agent.is_tablet  # True if mobile phone or tablet
        request.is_touch_capable = user_agent.is_touch_capable  # True if the client supports touch input
        request.is_pc = user_agent.is_pc  # True if the client is a PC
        request.is_bot = user_agent.is_bot  # True if the client is a bot

        # Pass the request to the next middleware or view and get the response
        response = self.get_response(request)

        # Return the processed response
        return response



from django.test import TestCase

from allianceauth.srp.form import SrpFleetUserRequestForm


class TestForms(TestCase):

    def test_allow_missing_trailing_slash_zkillboard(self):
        form = SrpFleetUserRequestForm(
            data = {
                "killboard_link": "https://zkillboard.com/kill/130429493",
                "additional_info": "Details",
            }
        )
        form.cleaned_data = {"killboard_link": "https://zkillboard.com/kill/130429493"}

        self.assertEqual("https://zkillboard.com/kill/130429493/", form.clean_killboard_link())

    def test_not_add_trailling_slash_kb_evetools(self):
        form = SrpFleetUserRequestForm(
            data = {
                "killboard_link": "https://kb.evetools.org/kill/130429493",
            }
        )
        form.cleaned_data = {"killboard_link": "https://kb.evetools.org/kill/130429493"}

        self.assertEqual("https://kb.evetools.org/kill/130429493", form.clean_killboard_link())



# Generated by Django 4.2.28 on 2026-02-16 12:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("timerboard", "0008_alter_timer_timer_type"),
    ]

    operations = [
        migrations.AlterField(
            model_name="timer",
            name="structure",
            field=models.CharField(
                choices=[
                    ("Other", "Other"),
                    ("Astrahus", "Astrahus"),
                    ("Athanor", "Athanor"),
                    ("Azbel", "Azbel"),
                    ("Pharolux Cyno Beacon", "Cyno Beacon"),
                    ("Tenebrex Cyno Jammer", "Cyno Jammer"),
                    ("Fortizar", "Fortizar"),
                    ("Ansiblex Jump Gate", "Jump Bridge"),
                    ("Keepstar", "Keepstar"),
                    ("Mercenary Den", "Mercenary Den"),
                    ("Metenox Moon Drill", "Moon Drill"),
                    ("Moon Mining Cycle", "Moon Mining Cycle"),
                    ("POCO", "POCO"),
                    ("POS[S]", "POS [S]"),
                    ("POS[M]", "POS [M]"),
                    ("POS[L]", "POS [L]"),
                    ("Raitaru", "Raitaru"),
                    ("Orbital Skyhook", "Skyhook"),
                    ("Sotiyo", "Sotiyo"),
                    ("I-HUB", "Sovereignty Hub"),
                    ("Tatara", "Tatara"),
                ],
                default="Other",
                max_length=254,
            ),
        ),
    ]



"""
Custom template tags for custom_css app
"""

# Alliance Auth Custom CSS
from allianceauth.custom_css.models import CustomCSS

# Django
from django.template.defaulttags import register
from django.utils.safestring import mark_safe


@register.simple_tag
def return_custom_css() -> str:
    """
    Template tag to load custom CSS

    :return:
    :rtype:
    """

    css = CustomCSS.get_solo()

    return (
        mark_safe(f"<style>{css.css_compressed}</style>") if css.css_compressed else ""
    )



# Generated by Django 4.2.25 on 2025-10-14 22:02

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("authentication", "0024_alter_userprofile_language"),
    ]

    operations = [
        migrations.AddField(
            model_name="userprofile",
            name="minimize_sidebar",
            field=models.BooleanField(
                default=False,
                help_text="Keep the sidebar menu minimized",
                verbose_name="Minimize Sidebar Menu",
            ),
        ),
    ]



# Generated by Django 1.10.2 on 2016-12-12 01:00

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('mumble', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelTable(
            name='mumbleuser',
            table=None,
        ),
    ]



"""
Test sentinel user
"""

import json
import re

# Django
from allianceauth.tests.auth_utils import AuthUtils
from django.test import RequestFactory, TestCase
from django.http import HttpRequest
# Alliance Auth
from allianceauth.framework.datatables import DataTablesView
from allianceauth.eveonline.models import EveCharacter

class TestView(DataTablesView):
    model=EveCharacter
    columns = [
        ("", "{{ row.character_id }}"),
        ("character_name", "{{ row.character_name }}"),
        ("corporation_name", "{{ row.corporation_name }}"),
        ("alliance_name", "{{ row.alliance_name }}"),
    ]

class TestDataTables(TestCase):
    def setUp(self):
        self.get_params = {
            'draw': '1',
            'columns[0][data]': '0',
            'columns[0][name]': '',
            'columns[0][searchable]': 'false',
            'columns[0][orderable]': 'false',
            'columns[0][search][value]': '',
            'columns[0][search][regex]': 'false',
            'columns[1][data]': '1',
            'columns[1][name]': '',
            'columns[1][searchable]': 'true',
            'columns[1][orderable]': 'true',
            'columns[1][search][value]': '',
            'columns[1][search][regex]': 'false',
            'columns[2][data]': '2',
            'columns[2][name]': '',
            'columns[2][searchable]': 'true',
            'columns[2][orderable]': 'false',
            'columns[2][search][value]': '',
            'columns[2][search][regex]': 'false',
            'columns[3][data]': '3',
            'columns[3][name]': '',
            'columns[3][searchable]': 'true',
            'columns[3][orderable]': 'true',
            'columns[3][search][value]': '',
            'columns[3][search][regex]': 'false',
            'order[0][column]': '1',
            'order[0][dir]': 'asc',
            'start': '0',
            'length': '10',
            'search[value]': '',
            'search[regex]': 'false',
            '_': '123456789'
        }


    @classmethod
    def setUpClass(cls) -> None:
        """
        Set up eve models
        """

        super().setUpClass()
        cls.factory = RequestFactory()

        cls.user = AuthUtils.create_user("bruce_wayne")
        cls.user.is_superuser = True
        cls.user.save()

        EveCharacter.objects.all().delete()
        for i in range(1,16):
            EveCharacter.objects.create(
                character_id=1000+i,
                character_name=f"{1000+i} - Test Character - {1000+i}",
                corporation_id=2000+i,
                corporation_name=f"{2000+i} - Test Corporation",
            )

        for i in range(16,21):
            EveCharacter.objects.create(
                character_id=1000+i,
                character_name=f"{1000+i} - Test Character - {1000+i}",
                corporation_id=2000+i,
                corporation_name=f"{2000+i} - Test Corporation",
                alliance_id=3000+i,
                alliance_name=f"{3000+i} - Test Alliance",
            )


    def test_view_default(self):
        self.client.force_login(self.user)
        request = self.factory.get('/fake-url/', data=self.get_params)
        response = TestView()
        response.setup(request)
        data = json.loads(response.get(request).content)["data"]
        self.assertEqual(data[0][0], "1001")
        self.assertEqual(data[9][0], "1010")

    def test_view_reverse_sort(self):
        self.get_params["order[0][dir]"] = "desc"
        self.client.force_login(self.user)
        request = self.factory.get('/fake-url/', data=self.get_params)
        response = TestView()
        response.setup(request)
        data = json.loads(response.get(request).content)["data"]
        self.assertEqual(data[0][0], "1020")
        self.assertEqual(data[9][0], "1011")

    def test_view_no_sort(self):
        self.get_params.pop("order[0][column]")
        self.get_params.pop("order[0][dir]")
        self.client.force_login(self.user)
        request = self.factory.get('/fake-url/', data=self.get_params)
        response = TestView()
        response.setup(request)
        data = json.loads(response.get(request).content)["data"]
        self.assertEqual(data[0][0], "1001")
        self.assertEqual(data[9][0], "1010")

    def test_view_non_sortable_sort(self):
        self.get_params["order[0][dir]"] = "desc"
        self.get_params["order[0][column]"] = "0"
        self.client.force_login(self.user)
        request = self.factory.get('/fake-url/', data=self.get_params)
        response = TestView()
        response.setup(request)
        data = json.loads(response.get(request).content)["data"]
        self.assertEqual(data[0][0], "1001")
        self.assertEqual(data[9][0], "1010")

    def test_view_20_rows(self):
        self.get_params["length"] = "20"
        self.client.force_login(self.user)
        request = self.factory.get('/fake-url/', data=self.get_params)
        response = TestView()
        response.setup(request)
        data = json.loads(response.get(request).content)["data"]
        self.assertEqual(data[0][0], "1001")
        self.assertEqual(data[19][0], "1020")

    def test_records_filtered(self):
        self.get_params["length"] = "20"
        self.client.force_login(self.user)
        request = self.factory.get('/fake-url/', data=self.get_params)
        response = TestView()
        response.setup(request)
        content = json.loads(response.get(request).content)
        self.assertEqual(content["recordsFiltered"], 20)
        self.assertEqual(content["recordsTotal"], 20)

    def test_view_global_search(self):
        self.get_params["search[value]"] = "1020"
        self.client.force_login(self.user)
        request = self.factory.get('/fake-url/', data=self.get_params)
        response = TestView()
        response.setup(request)
        data = json.loads(response.get(request).content)["data"]
        self.assertEqual(len(data), 1)
        self.assertEqual(data[0][0], "1020")

    def test_view_col_1_search(self):
        self.get_params["columns[1][search][value]"] = "1020"
        self.client.force_login(self.user)
        request = self.factory.get('/fake-url/', data=self.get_params)
        response = TestView()
        response.setup(request)
        data = json.loads(response.get(request).content)["data"]
        self.assertEqual(len(data), 1)
        self.assertEqual(data[0][0], "1020")

    def test_view_col_1_search_empty(self):
        self.get_params["columns[1][search][value]"] = "zzz"

        self.client.force_login(self.user)
        request = self.factory.get('/fake-url/', data=self.get_params)
        response = TestView()
        response.setup(request)
        data = json.loads(response.get(request).content)["data"]
        self.assertEqual(len(data), 0)

    def test_view_cc_3_search_empty(self):
        self.get_params["columns[3][columnControl][search][value]"] = ""
        self.get_params["columns[3][columnControl][search][logic]"] = "empty"
        self.get_params["columns[3][columnControl][search][type]"] = "text"
        self.get_params["length"] = "20"

        self.client.force_login(self.user)
        request = self.factory.get('/fake-url/', data=self.get_params)
        response = TestView()
        response.setup(request)
        data = json.loads(response.get(request).content)["data"]
        self.assertEqual(len(data), 15)

    def test_view_cc_3_search_not_empty(self):
        self.get_params["columns[3][columnControl][search][value]"] = ""
        self.get_params["columns[3][columnControl][search][logic]"] = "notEmpty"
        self.get_params["columns[3][columnControl][search][type]"] = "text"
        self.client.force_login(self.user)
        request = self.factory.get('/fake-url/', data=self.get_params)
        response = TestView()
        response.setup(request)
        data = json.loads(response.get(request).content)["data"]
        self.assertEqual(len(data), 5)

    def test_view_cc_1_search_ends_with(self):
        self.get_params["columns[1][columnControl][search][value]"] = "9"
        self.get_params["columns[1][columnControl][search][logic]"] = "ends"
        self.get_params["columns[1][columnControl][search][type]"] = "text"
        self.client.force_login(self.user)
        request = self.factory.get('/fake-url/', data=self.get_params)
        response = TestView()
        response.setup(request)
        data = json.loads(response.get(request).content)["data"]
        self.assertEqual(len(data), 2)

    def test_view_cc_1_search_starts_with(self):
        self.get_params["columns[1][columnControl][search][value]"] = "1009"
        self.get_params["columns[1][columnControl][search][logic]"] = "starts"
        self.get_params["columns[1][columnControl][search][type]"] = "text"
        self.client.force_login(self.user)
        request = self.factory.get('/fake-url/', data=self.get_params)
        response = TestView()
        response.setup(request)
        data = json.loads(response.get(request).content)["data"]
        self.assertEqual(len(data), 1)

    def test_view_cc_1_search_not_contains(self):
        self.get_params["columns[1][columnControl][search][value]"] = "100"
        self.get_params["columns[1][columnControl][search][logic]"] = "notContains"
        self.get_params["columns[1][columnControl][search][type]"] = "text"
        self.get_params["length"] = "20"
        self.client.force_login(self.user)
        request = self.factory.get('/fake-url/', data=self.get_params)
        response = TestView()
        response.setup(request)
        data = json.loads(response.get(request).content)["data"]
        self.assertEqual(len(data), 11)

    def test_view_cc_1_search_contains(self):
        self.get_params["columns[1][columnControl][search][value]"] = "100"
        self.get_params["columns[1][columnControl][search][logic]"] = "contains"
        self.get_params["columns[1][columnControl][search][type]"] = "text"
        self.get_params["length"] = "20"
        self.client.force_login(self.user)
        request = self.factory.get('/fake-url/', data=self.get_params)
        response = TestView()
        response.setup(request)
        data = json.loads(response.get(request).content)["data"]
        self.assertEqual(len(data), 9)

    def test_view_cc_1_search_equal(self):
        self.get_params["columns[1][columnControl][search][value]"] = "1001 - Test Character - 1001"
        self.get_params["columns[1][columnControl][search][logic]"] = "equal"
        self.get_params["columns[1][columnControl][search][type]"] = "text"
        self.get_params["length"] = "20"
        self.client.force_login(self.user)
        request = self.factory.get('/fake-url/', data=self.get_params)
        response = TestView()
        response.setup(request)
        data = json.loads(response.get(request).content)["data"]
        self.assertEqual(len(data), 1)

    def test_view_cc_1_search_not_equal(self):
        self.get_params["columns[1][columnControl][search][value]"] = "1001 - Test Character - 1001"
        self.get_params["columns[1][columnControl][search][logic]"] = "notEqual"
        self.get_params["columns[1][columnControl][search][type]"] = "text"
        self.get_params["length"] = "20"
        self.client.force_login(self.user)
        request = self.factory.get('/fake-url/', data=self.get_params)
        response = TestView()
        response.setup(request)
        data = json.loads(response.get(request).content)["data"]
        self.assertEqual(len(data), 19)

    def test_view_cc_no_pagination(self):
        self.get_params["length"] = "-1"
        self.client.force_login(self.user)
        request = self.factory.get('/fake-url/', data=self.get_params)
        response = TestView()
        response.setup(request)
        data = json.loads(response.get(request).content)["data"]
        self.assertEqual(len(data), 20)



# Generated by Django 4.2.28 on 2026-02-14 13:12

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ("custom_css", "0001_initial"),
    ]

    operations = [
        migrations.AddField(
            model_name="customcss",
            name="css_compressed",
            field=models.TextField(
                blank=True,
                help_text="This field is automatically generated and should not be edited.",
                null=True,
                verbose_name="Compressed custom CSS",
            ),
        ),
    ]



from collections import defaultdict
import re
from typing import List

from django.db.models import Model, Q
from django.http import HttpRequest, JsonResponse
from django.template import Context, Template
from django.template.loader import render_to_string
from django.views import View

from allianceauth.services.hooks import get_extension_logger


logger = get_extension_logger(__name__)


class nested_param_dict(dict):
    """
    Helper to create infinite depth default dicts for setting from params
    """
    def __setitem__(self, item, value):
        if "." in item:
            head, path = item.split(".", 1)
            try:
                head = int(head)
            except ValueError:
                pass
            obj = self.setdefault(head, nested_param_dict())
            obj[path] = value
        else:
            super().__setitem__(item, value)


def defaultdict_to_dict(d):
    """
    Helper to convert default dict back to dict
    """
    if isinstance(d, defaultdict):
        d = {k: defaultdict_to_dict(v) for k, v in d.items()}
    return d


class DataTablesView(View):

    model: Model = None
    columns: List[tuple] = []

    def get_model_qs(self, request: HttpRequest, *args, **kwargs):
        return self.model.objects

    def filter_qs(self, table_conf: dict):
        # Search
        filter_qs = Q()
        for id, c in table_conf["columns"].items():
            _c = self.columns[int(id)][0]
            if c.get("searchable", False) and len(_c) > 0:
                if c.get("columnControl", False):
                    _sv = str(c["columnControl"]["search"]["value"])
                    """contains, equal, ends, starts, empty"""
                    _logic = str(c["columnControl"]["search"]["logic"])
                    """text, date, num"""
                    _type = str(c["columnControl"]["search"]["type"])
                    if _type == "text":
                        if _logic == "empty":
                            filter_qs &= Q(**{f'{_c}': ""})
                        elif len(_sv) > 0:
                            if _logic == "contains":
                                filter_qs &= Q(**{f'{_c}__icontains': _sv})
                            elif _logic == "starts":
                                filter_qs &= Q(**{f'{_c}__istartswith': _sv})
                            elif _logic == "ends":
                                filter_qs &= Q(**{f'{_c}__iendswith': _sv})
                            elif _logic == "equal":
                                filter_qs &= Q(**{f'{_c}': _sv})
                    elif _type == "num":
                        if _logic == "empty":
                            filter_qs &= Q(**{f'{_c}__isnull': True})
                        elif len(_sv) > 0:
                            try:
                                if _logic == "greater":
                                    filter_qs &= Q(**{f'{_c}__gt': float(_sv)})
                                elif _logic == "less":
                                    filter_qs &= Q(**{f'{_c}__lt': float(_sv)})
                                elif _logic == "greaterOrEqual":
                                    filter_qs &= Q(**{f'{_c}__gte': float(_sv)})
                                elif _logic == "lessOrEqual":
                                    filter_qs &= Q(**{f'{_c}__lte': float(_sv)})
                                elif _logic == "equal":
                                    filter_qs &= Q(**{f'{_c}': float(_sv)})
                            except ValueError:
                                pass
                else:
                    _sv = str(c["search"]["value"])
                    if len(_sv) > 0:
                        if c["search"]["regex"]:
                            filter_qs |= Q(**{f'{_c}__iregex': _sv})
                        else:
                            filter_qs |= Q(**{f'{_c}__icontains': _sv})
                _gsv = str(table_conf["search"]["value"])
                if len(_gsv) > 0:
                    filter_qs |= Q(**{f'{_c}__icontains': _gsv})
        return filter_qs

    def except_qs(self, table_conf: dict):
        # Search
        except_qs = Q()
        for id, c in table_conf["columns"].items():
            _c = self.columns[int(id)][0]
            if c.get("searchable", False) and len(_c) > 0:
                if c.get("columnControl", False):
                    _sv = str(c["columnControl"]["search"]["value"])
                    """notContains, notEqual, notEmpty"""
                    _logic = str(c["columnControl"]["search"]["logic"])
                    """text, date, num"""
                    _type = str(c["columnControl"]["search"]["type"])
                    if _type == "text":
                        if _logic == "notEmpty":
                            except_qs |= Q(**{f'{_c}': ""})
                        elif len(_sv) > 0:
                            if _logic == "notContains":
                                except_qs |= Q(**{f'{_c}__icontains': _sv})
                            elif _logic == "notEqual":
                                except_qs |= Q(**{f'{_c}': _sv})
                    elif _type == "num":
                        if _logic == "notEmpty":
                            except_qs |= Q(**{f'{_c}__isnull': False})
                        elif len(_sv) > 0:
                            if _logic == "notEqual":
                                try:
                                    except_qs |= Q(**{f'{_c}': float(_sv)})
                                except ValueError:
                                    pass
        return except_qs

    def get_table_config(self, get: dict):
        _cols = nested_param_dict()
        for c, v in get.items():
            _keys = [_k for _k in c.replace("]", "").split("[")]
            _v = v
            if v in ["true", "false"]:
                _v = _v == "true"
            else:
                try:
                    _v = int(_v)
                except ValueError:
                    pass # not an integer
            _cols[".".join(_keys)] = _v
        return defaultdict_to_dict(_cols)

    def get_order(self, table_conf: dict):
        order = []
        for oc, od in table_conf.get("order", {}).items():
            _c = table_conf["columns"][od["column"]]
            if _c["orderable"]:
                if od["dir"] == "desc":
                    order.append("-" + self.columns[int(od["column"])][0])
                else:
                    order.append(self.columns[int(od["column"])][0])
        return order

    def render_template(self, request: HttpRequest, template: str, ctx: dict):
        if "{{" in template:
            _template = Template(template)
            return _template.render(Context(ctx))
        else:
            return render_to_string(
                template,
                ctx,
                request
            )

    def get(self, request: HttpRequest, *args, **kwargs):
        table_conf = self.get_table_config(request.GET)
        draw = int(table_conf["draw"])
        start = int(table_conf["start"])
        length = int(table_conf["length"])
        if length <= 0:
            logger.warning(
                "Using no pagination is not recommended for server side rendered datatables"
            )
        limit = start + length


        # Build response rows
        items = []
        qs = self.get_model_qs(
            request,
            *args,
            **kwargs
        ).filter(
            self.filter_qs(table_conf)
        ).exclude(
            self.except_qs(table_conf)
        ).order_by(
            *self.get_order(table_conf)
        )

        # Get the count after filtering
        qs_count = qs.count()

        # build output
        if length > 0:
            qs = qs[start:limit]

        for row in qs:
            ctx = {"row": row}
            row = []
            for t in self.columns:
                row.append(self.render_template(request, t[1], ctx))
            items.append(row)

        # Build our output dict
        datatables_data = {}
        datatables_data['draw'] = draw
        datatables_data['recordsTotal'] = self.get_model_qs(request, *args, **kwargs).all().count()
        datatables_data['recordsFiltered'] = qs_count
        datatables_data['data'] = items

        return JsonResponse(datatables_data)



from django.test import TestCase

from allianceauth.custom_css.models import CustomCSS


class TestCompressCSS(TestCase):
    """
    Test cases for the compress_css method of the CustomCSS model
    """

    def test_compresses_css_by_removing_comments(self):
        """
        Test that the compress_css method removes CSS comments

        :return:
        :rtype:
        """

        instance = CustomCSS(css="body { color: red; } /* This is a comment */")
        compressed = instance.compress_css()

        self.assertNotIn("/* This is a comment */", compressed)

    def test_compresses_css_by_removing_extra_spaces(self):
        """
        Test that the compress_css method removes extra spaces

        :return:
        :rtype:
        """

        instance = CustomCSS(css="body    {    color:    red;    }")
        compressed = instance.compress_css()

        self.assertEqual(compressed, "body{color:red}")

    def test_compresses_css_by_shortening_hex_colors(self):
        """
        Test that the compress_css method shortens hex colors when possible

        :return:
        :rtype:
        """

        instance = CustomCSS(css="body { color: #aabbcc; }")
        compressed = instance.compress_css()

        self.assertIn("#abc", compressed)

    def test_retains_url_without_quotes(self):
        """
        Test that the compress_css method retains url() values without quotes

        :return:
        :rtype:
        """

        instance = CustomCSS(css="body { background: url('image.png'); }")
        compressed = instance.compress_css()

        self.assertIn("url(image.png)", compressed)

    def test_handles_empty_css_gracefully(self):
        """
        Test that the compress_css method handles empty CSS gracefully

        :return:
        :rtype:
        """

        instance = CustomCSS(css="")
        compressed = instance.compress_css()

        self.assertEqual(compressed, "")

    def test_handles_invalid_css_gracefully(self):
        """
        Test that the compress_css method handles invalid CSS gracefully

        :return:
        :rtype:
        """

        instance = CustomCSS(css="invalid-css")
        compressed = instance.compress_css()

        self.assertEqual(compressed, "")


class TestSave(TestCase):
    """
    Test cases for the save method of the CustomCSS model
    """

    def test_saves_instance_with_compressed_css(self):
        """
        Test that the save method saves the instance with compressed CSS

        :return:
        :rtype:
        """

        instance = CustomCSS(css="body { color: red; } /* comment */")
        instance.save()

        self.assertEqual(instance.css_compressed, "body{color:red}")

    def test_overwrites_primary_key_on_save(self):
        """
        Test that the save method overwrites the primary key to 1

        :return:
        :rtype:
        """

        instance = CustomCSS(css="body { color: red; }")
        instance.save()

        self.assertEqual(instance.pk, 1)

    def test_handles_save_with_empty_css(self):
        """
        Test that the save method handles saving an instance with empty CSS

        :return:
        :rtype:
        """

        instance = CustomCSS(css="")
        instance.save()

        self.assertEqual(instance.css_compressed, "")

    def test_retains_existing_compressed_css_on_save(self):
        """
        Test that the save method retains existing compressed CSS if the css field is not changed

        :return:
        :rtype:
        """

        instance = CustomCSS(css="body { color: blue; }")
        instance.css_compressed = "body{color:blue}"
        instance.save()

        self.assertEqual(instance.css_compressed, "body{color:blue}")


class TestModelNameAndfields(TestCase):
    """
    Test cases for the model name and fields of the CustomCSS model
    """

    def test_model_name(self):
        """
        Test that the model name is correct

        :return:
        :rtype:
        """

        instance = CustomCSS()
        self.assertEqual(str(instance), "Custom CSS")

    def test_fields(self):
        """
        Test that the model fields are correct

        :return:
        :rtype:
        """

        instance = CustomCSS()
        self.assertTrue(hasattr(instance, "css"))
        self.assertTrue(hasattr(instance, "css_compressed"))
        self.assertTrue(hasattr(instance, "timestamp"))

    def test_allows_null_and_blank_values_for_css_fields(self):
        """
        Test that the css and css_compressed fields allow null and blank values

        :return:
        :rtype:
        """

        instance = CustomCSS(css=None, css_compressed=None)
        instance.full_clean()  # Should not raise ValidationError

    def test_updates_timestamp_on_save(self):
        """
        Test that the timestamp field is updated on save

        :return:
        :rtype:
        """

        instance = CustomCSS(css="body { color: red; }")
        instance.save()

        previous_timestamp = instance.timestamp

        instance.css = "body { color: blue; }"
        instance.save()

        self.assertNotEqual(instance.timestamp, previous_timestamp)



"""
Tests for the custom_css template tag.
"""

from django.test import TestCase

from allianceauth.custom_css.models import CustomCSS
from allianceauth.custom_css.templatetags.custom_css import return_custom_css


class TestCustomCssTemplateTag(TestCase):
    """
    Test the custom_css template tag
    """

    def test_returns_empty_string_when_css_compressed_is_empty(self):
        """
        Test that the custom_css template tag returns an empty string when css_compressed is empty.

        :return:
        :rtype:
        """

        CustomCSS.objects.create(
            pk=CustomCSS.singleton_instance_id, css="", css_compressed=""
        )
        result = return_custom_css()

        self.assertEqual(result, "")

    def test_returns_style_tag_with_compressed_css(self):
        """
        Test that the custom_css template tag returns a style tag with the compressed CSS when css_compressed is not empty.

        :return:
        :rtype:
        """

        CustomCSS.objects.create(
            pk=CustomCSS.singleton_instance_id,
            css="body { color: red }",
            css_compressed="body{color:red}",
        )
        result = return_custom_css()

        self.assertEqual(result, "<style>body{color:red}</style>")
